# -*- coding: utf-8 -*-
'''
Учитель информатики Иван Васильевич составляет для школьников 24-ю задачу
и генерирует случайную строку из length=10_000 символов,
число различных символов k (например, при k=3 строка может состоять только из символов A, B или C).
Найдите вероятность того, что в данной строке будет n=10 или
более подряд идущих одинаковых символов.
Ответ дайте в процентах, округлив до двух знаков после запятой.
'''
from time import time
from itertools import product


LENGTH = 10_000  # длина строки
N = 10  # ищем вероятность встретить N одинаковых букв подряд
K = 3   # количество различных используемых букв


# Время работы O(k^length * N * K)
def slow_solution():
    global LENGTH, N, K
    alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    total = 0
    good = 0
    for x in product(alphabet[:K], repeat=LENGTH):
        total += 1
        word = ''.join(x)
        for i in alphabet[:K]:
            if i * N in word:
                good += 1
                break
    return good / total


# Время работы O(n * length)
def good_solution():
    global LENGTH, N, K

    # Стартуем от однобуквенных слов!
    start = time()
    a = [0] * (N + 1)
    a[1] = K  # существует ровно K различных однобуквенных слов

    # А теперь динамика!
    # Пусть a[i], i < N - количество слов, у которых не менее i одинаковых букв в конце
    # a[N] - слова, в которых есть не менее N одинаковых букв подряд

    # от x-буквенных слов переходим к (x + 1)-буквенным словам
    # Тогда a[1] = (K - 1) * (количество всех x-буквенных слов, к которым приписали букву, отличную от самой последней,
    # кроме слов, у которых на конце уже N одинаковых букв)
    # a[i] = a[i - 1]  (количество всех x-буквенных слов, у которы на конце было (i-1) одинаковая буква,
    # дописали ещё одну такую же букву
    # a[N] = k * a[N] + a[N - 1]  к словам, в которых уже попалось N одинаковых букв подряд, можно приписывать что угодно
    # плюс в категорию хороших попали a[N-1] слов, к которым приписали одну такую же букву
    for x in range(2, LENGTH + 1):
        new_a = [0] * (N + 1)
        new_a[1] = (K - 1) * sum(a[1:N])
        new_a[N] = K * a[N] + a[N - 1]
        for i in range(2, N):
            new_a[i] = a[i - 1]
        a = new_a
        # проверим, что количество слов в пересчитанном списке равно k ** x
        # print(x, a, sum(a), K ** x)

    stop = time()
    probability = a[N] / sum(a)  # искомая вероятность
    duration = stop - start  # время работы программы
    # return a[N], sum(a), duration
    return probability, duration


if LENGTH <= 10 and K <= 15:
    print("Искомая вероятность:", slow_solution())
probability, duration = good_solution()
print("Искомая вероятность:", probability)
print("Время работы:", duration)